
import string
from random import sample, randint
from os.path import join


printable_line = string.printable.replace('\n', '').replace('\r', '')


def get_random_word():
	"""
	:return: random alphanumeric word of 2-16 characters
	"""
	return unicode(''.join(sample(string.letters + string.digits, randint(2, 16))))


def get_random_string():
	"""
	:return: random string of 2-16 printable characters, without newlines
	"""
	return unicode(''.join(sample(printable_line, randint(2, 16))))


def create_file(dir_path):
	"""
	Creates a small random file, for testing.

	:param dir_path: (temporary) dir to create the file
	:return: full path to the file
	"""
	file_path = join(dir_path, get_random_word()).rstrip('/') + '.' + get_random_word()[:3]
	fh = open(file_path, 'w+')
	fh.write('\n'.join(get_random_string() for line in range(100)))
	fh.close()
	return file_path


def dict_round_floats(dicti, decim = 6):
	"""
	Go through a dict and round all floats (in keys and values) to a specific number of decimal places, to make it
	possible to compare a dict that has been saved and loaded, or some math applied.

	:return: no return value; **substitution happens in-place**!
	"""
	for key, value in dicti.items():
		if isinstance(key, float):
			''' this also matches for numpy.float64 etc '''
			del dicti[key]
			key = round(key, decim)
			dicti[key] = value
		if isinstance(value, float):
			dicti[key] = round(value, decim)






"""
Routines related to text input/output
"""

import sys, os


def clear_stdout(lines = 1, stream = sys.stdout, flush = True):
	"""
	Uses escape characters to move back number of lines and empty them

	To write something on the empty lines, use :func:sys.stdout.write() and :func:sys.stdout.flush()

	:param lines: the number of lines to clear
	:param stream: by default, stdout is cleared, but this can be replaced by another stream that uses these \
	escape characters (stderr comes to mind)
	:param flush: by default, uses flush after writing, but canbe overriden if you will do it yourself soon after
	"""
	stream.write('\033[F\r\x1b[K' * lines)
	if flush:
		stream.flush()


def reprint(txt, lines = 1, stream = sys.stdout):
	"""
	Removes lines from stdout and prints the requested text txt instead of them

	:param txt: the text you want to print; works best if no more than `lines` lines
	:param stream: see ``clear_stdout``
	:param lines: the number of lines you want to use; \
	this amount is cleared and if txt is shorter, the remainder is prepended

	For example for status monitoring:

	1. print N newlines
	2. do something or check something until there is news
	3. use ``reprint`` to print the as most N lines update using lines = N
	4. repeat step 2 and 3 until task is done
	5. if you want to remove the last status message, call clear_stdout using lines = N
	"""
	clear_stdout(lines = lines, flush = False)
	line_count = len(txt.splitlines())
	stream.write(
		os.linesep * (lines - line_count) +
		txt +
		os.linesep
	)
	stream.flush()


def add_linebreaks(text, max_len=80):
	"""
	Add linebreaks on whitespace such that no line is longer than `max_len`, unless it contains a single word that's longer.
	
	There are probably way faster methods, but this is simple and works.
	"""
	br_text = ''
	len_cnt = 0
	for word in text.split(' '):
		len_cnt += len(word) + 1
		if len_cnt > max_len:
			len_cnt = len(word)
			br_text += '\n' + word
		else:
			br_text += ' ' + word
	return br_text[1:]






"""
Routines related to changing text, e.g. substitution or escape for some format
"""

from re import escape as re_escape, compile


#todo: unit test
def tex_escape(text):
	"""
	:param text: a plain text message
	:return: the message escaped to appear correctly in LaTeX
	"""
	conv = {
		'&': r'\&',
		'%': r'\%',
		'$': r'\$',
		'#': r'\#',
		'_': r'\_',
		'{': r'\{',
		'}': r'\}',
		'~': r'\textasciitilde{}',
		'^': r'\^{}',
		'\\': r'\textbackslash{}',
		'<': r'\textless',
		'>': r'\textgreater',
	}
	return sub_dict(text, conv)


def sub_dict(text, subst):
	"""
	Replace all instances of multiple words in a string. Prevents problems with overlapping keys and substitutions.

	:param subst: something that behaves like a dictionary, where each key is converted to it's value
	"""
	regex = compile('|'.join(re_escape(unicode(key)) for key in sorted(subst.keys(), key = lambda item: - len(item))))
	return regex.sub(lambda match: subst[match.group()], unicode(text))






"""
These functions should perhaps be included, but have not been sufficiently tested yet
"""

from numpy import prod


# from numerical_recipes/4/io
	# maybe not, perhaps realistically the grid would not be completely full and thus stored differently
def savegrid(filename, data, dims, format='%.18e', delims=['\t', '\n']):
	"""
	Save a 1D vector representing a grid to a file with given formatting and delimiters.

	The most typical example would be a 2D wavefunction stored in a 1D vector for numerical reasons.
	"""
	if len(delims) == 1:
		delims = [delims] * len(dims)
	elif len(delims) < len(dims):
		raise AssertionError('need a delimiter for each dimension! [%d]' % len(delims))
	assert len(data) == prod(dims), 'data should be a (1D) vector with a value for each grid point in dims'
	with open(filename, 'w+') as fh:
		for k, dim, delim in zip(range(len(dims)), dims, delims):
			print('dimension')
			for val in dim:
				fh.write('%s%s' % (format % val, delim))
			fh.write('\n')







from numpy import roll


def translate(arr, shifts, fill_value = 0):
	"""
	For a numpy array (e.g. image), translate all values by an integer vector.

	:arg arr: The ndarray to shift (remains unaffected).
	:arg shifts: The sequence of shift distances (integers).
	:return: shifted array

	Compare roll and pad.
	"""
	#todo: unit tests
	Ns = arr.shape
	assert len(Ns) == len(shifts), 'Provide a shift distance for each axis'
	for k, shift in enumerate(shifts):
		if shift != 0:
			arr = roll(arr, shift = shift, axis = k)
			s = [slice(None)] * len(shifts)
			s[k] = slice(0, shift) if shift > 0 else slice(shift, Ns[k])
			arr[s] = fill_value
	return arr






from threading import Thread
import urllib2


def async_forget_thread(func, *args, **kwargs):
	"""
	Very simply async decorator, intended for writing things while continuing the program. No output is sent back.

	* Uses non-daemon mode, which causes the program to keep running until this thread exits.
	* Using threading, so runs on the same code due to Python global interpreter lock.
	"""
	#Thread(target = func, args = args, kwargs = kwargs, daemon = False).start()
	Thread(target = func, args = args, kwargs = kwargs).start()
	#todo: tested only superficially


def fetch_urls(urls):
	"""
	Fetch urls in parllel, then wait until they're all done.

	@return: content specified by the urls in the same order as the urls

	http://stackoverflow.com/questions/16181121/python-very-simple-multithreading-parallel-url-fetching-without-queue
	"""
	results = [None] * len(urls)
	def fetch_url(nr, url):
		results[nr] = urllib2.urlopen(url).read()
	threads = [Thread(target = fetch_url, args = (nr, url)) for nr, url in enumerate(urls)]
	for thread in threads: thread.start()
	for thread in threads: thread.join()
	return results






"""
N-dimensional grid.
"""

from numpy import prod
from sys import stderr


stderr.write('These grid functions are deprecated since numpy offers the same functionality using unravel_index and ravel_multi_index.')


def to_coordinate(index, dims):
	"""
	Convert from 'element N' to 'result matrix coordinate [x, y, z]'.
	"""
	assert 0 <= index < remaining_dims(dims)[0], 'The index {0:d} is out of bounds (the grid has {1:d} elements)'.format(index, remaining_dims(dims)[0])
	return tuple(index // remaining_dims(dims)[k + 1] % dim for k, dim in enumerate(dims))


def from_coordinate(coordinate, dims):
	"""
	Convert from 'result matrix coordinate [x, y, z]' to 'element N'.
	"""
	for coord, dim in zip(coordinate, dims):
		assert 0 <= coord <  dim
	return sum(tuple(coordinate[k] * remaining_dims(dims)[k + 1] for k in range(len(dims))))


def remaining_dims(dims):
	remaining_dims.CACHE = getattr(remaining_dims, 'CACHE', {})
	remaining_dims.CACHE[dims] = remaining_dims.CACHE.get(dims, tuple(prod(dims[k:], dtype = int) for k in range(len(dims) + 1)))
	return remaining_dims.CACHE[dims]






"""
System related utilities (e.g. filesystem)
"""

from os import symlink, makedirs
from shutil import copyfile
from errno import EEXIST
from os.path import exists, isdir


def link_else_copy(filename_from, filename_to):
	"""
	Try to create a symbolic link 'copy' of a file (almost no space use).

	If that doesn't work then really copy them (files are independent).

	:param filename_from: the existing file
	:param filename_to: the desired new file
	:return: True if linking worked, False otherwise
	:raise: OSError if copy fails

	This command was switched from hard to soft links because it's clearer and works across filesystem boundaries.
	"""
	try:
		symlink(filename_from, filename_to)
		return True
	except OSError:
		copyfile(filename_from, filename_to)
		return False


def mkdirp(dir_path, mode=None):
	"""
	Creates all the directories on dir_path if they do not exist.

	Like ``mkdir -p`` in shell.

	:param dir_path: path whose components will be created as directories
	:raise: OSError for unexpected problems and if dir_path is something other than a directory;\
	no error if directory already exists
	"""
	if exists(dir_path) and not isdir(dir_path):
		raise OSError('"%s" already exists, but is not a directory' % dir_path)
	try:
		if mode is None:
			makedirs(dir_path)
		else:
			makedirs(dir_path, mode=mode)
	except OSError as err:
		if err.errno != EEXIST:
			raise


"""
	NOTE: if you add something, write a test!
"""







from numpy import eye, float64


def laplacian(N, dtype = float64):
	"""
	Create a del2 laplacian matrix of dimension N.

	E.g. for N = 4:

	.. math::

		\begin{matrix}
		 2 & -1 &  0 &  0 \\\\
		-1 &  2 & -1 &  0 \\\\
		 0 & -1 &  2 & -1 \\\\
		 0 &  0 & -1 &  2 \\\\
		\end{matrix}

	Used for numerical differentiation.

	In many cases, :ref: scipy.ndimage.filters.laplace, which calculates the derivative of a vector,
	is faster (but might be slightly different at boundaries).

	Note that this is not the graph theory laplacian matrix.
	"""
	return 2 * eye(N, k = 0, dtype = dtype) + \
			 - eye(N, k = -1, dtype = dtype) + \
			 - eye(N, k = 1, dtype = dtype)

"""
	NOTE: if you add something, write a test!
"""




"""
Utility methods for storing and retrieving data structures.
"""

from genericpath import getmtime
from time import time
from numpy import save, load, savetxt, loadtxt, float64
from json import dumps, loads
import gzip
from os.path import dirname
from bardeen.system import mkdirp
from collections import Mapping
from os.path import exists


"""
	General
"""


class EOAgeError(IOError):
	""" The file is older than the provided maximum age. """


def fail_if_too_old(filename, max_age = None):
	if not exists(filename):
		raise IOError('file "%s" could not be found' % filename)
	if max_age is not None:
		if time() - getmtime(filename) > max_age:
			raise EOAgeError('file "%s" could not be loaded because it is %ds old, which is more than the maximum age %ds.' % (filename, time() - getmtime(filename), max_age))


"""
	Strings
"""
#todo: unit tests for strings
#todo: unit tests for all max age


def store_str(string, filename, compressed = False):
	"""
	Store a string to a file.
	"""
	mkdirp(dirname(filename))
	if compressed:
		fh = gzip.open(filename, 'w+')
	else:
		fh = open(filename, 'w+')
	fh.write(string)
	fh.close()


def load_str(filename, max_age = None, compressed = False):
	"""
	Load a string from a file.
	"""
	fail_if_too_old(filename, max_age)
	if compressed:
		fh = gzip.open(filename, 'r')
	else:
		fh = open(filename, 'r')
	string = fh.read()
	fh.close()
	return string


"""
	numpy arrays
"""


def store_array(arr, filename, header = ''):
	"""
	Store a numpy array in a format readable by many programs
	(whitespace delimitered plain text with # header)

	:param arr: numpy array
	:param filename: (string) path to store the file; directory is created if it does not exist
	"""
	mkdirp(dirname(filename))
	if 'int' in unicode(arr.dtype):
		fmt = '%18d'
	else:
		fmt = '%.18e'
	savetxt(filename, arr, fmt = fmt, delimiter = '\t', newline = '\n', header = header)


def load_array(filename, dtype = float64, delimiter = '\t', max_age = None):
	"""
	Load a tab-delimitered array, e.g. as stored by :ref: store_array

	:param filename: (string) path to the file
	:param max_age: (None or number in seconds) if set, file will only be loaded if the file is newer than max_age

	:raises IOError: raied if file can't be opened
	:raises IOAgeError: subclass of IOError, raised if file exists but is older than max_age
	"""
	fail_if_too_old(filename, max_age)
	return loadtxt(filename, dtype = dtype, delimiter = delimiter)


def store_array_bin(arr, filename):
	"""
	Store as binary, single-array numpy file

	:param arr: numpy array
	:param filename: (string) path to store the file; directory is created if it does not exist

	extension .npy is strongly recommended
	"""
	#todo: maybe gzip later
	mkdirp(dirname(filename))
	save(filename, arr)


def load_array_bin(filename, max_age = None):
	"""
	Load a binary, single-array numpy file

	:param filename: (string) path to the file
	:param max_age: (None or number in seconds) if set, file will only be loaded if the file is newer than max_age

	:raises IOError: raied if file can't be opened
	:raises IOAgeError: subclass of IOError, raised if file exists but is older than max_age
	"""
	fail_if_too_old(filename, max_age)
	return load(filename)


"""
	dictionaries
"""


def store_conf(dicti, filename, header = None):
	"""
	Store a python dictionary with simple int, float, unicode elements
	as a easily readable config file.

	:param dicti: (dict) dictionary to store; keys (config options) are not allowed to contain whitespace
	:param filename: (string) path to store the file; directory is created if it does not exist
	:param header: (string/None) if string, added at the beginning of the file

	* ``None`` is allowed for values but not keys
	* Note that floats will be slightly different when loaded
	* Note that string '0123' will be loaded as integer 123
	"""
	assert isinstance(dicti, Mapping), 'argument is not a dictonary-like object'
	mkdirp(dirname(filename))
	with open(filename, 'w+') as fh:
		if header:
			for line in header.splitlines():
				fh.write('# %s\n' % line)
		for key, value in dicti.items():
			if isinstance(key, basestring):
				if ' ' in key or '\t' in key or '\n' in key or '\r' in key:
					raise KeyError('config dict can not be stored if keys contain whitespace')
			else:
				try:
					float(key)
				except:
					raise KeyError('key %s is not a string or number' % key)
			if isinstance(value, basestring):
				if '\n' in value or '\r' in value:
					raise ValueError('config dict can not be stored if values contain newlines')
			else:
				try:
					float(value)
				except:
					raise KeyError('value %s is not a string or number' % value)
			fh.write('%s\t%s\n' % (
				key,
				value,
			))


def load_conf(filename, max_age = None):
	"""
	Load a whitespace-delimitered config file, e.g. as stored by :ref: store_conf.

	:param filename: (string) path to the file
	:param max_age: (None or number in seconds) if set, file will only be loaded if the file is newer than max_age

	:raises IOError: raied if file can't be opened
	:raises IOAgeError: subclass of IOError, raised if file exists but is older than max_age
	"""
	fail_if_too_old(filename, max_age)
	with open(filename, 'r') as fh:
		dicti = {}
		''' http://stackoverflow.com/questions/15233340/getting-rid-of-n-when-using-readlines '''
		for nr, line in enumerate(fh.read().splitlines()):
			if not line.startswith('#'):
				try:
					raw_key, raw_value = line.split(None, 1)
				except ValueError:
					raise ValueError('config file %s inproperly configured in line %d: no delimiter whitespace (make sure comments start with #)' % (filename, nr))
				try:
					key = int(raw_key)
				except ValueError:
					try:
						key = float(raw_key)
					except ValueError:
						key = unicode(raw_key)
				try:
					value = int(raw_value)
				except ValueError:
					try:
						value = float(raw_value)
					except ValueError:
						value = unicode(raw_value)
				dicti[key] = value
	return dicti


def store_dict(dicti, filename, compressed = False):
	"""
	Store a dictionary as json.

	:param dicti: (dict) dictionary to store
	:param filename: (string) path to store the file; .gz is appended if compressed; directory is created if it does not exist
	:param compressed: if True, the file is gzipped and .gz is appended to the filename

	* Contrary to :ref: store_conf, whitespace in keys is allowed
	* See notes at :ref: store_conf
	"""
	assert isinstance(dicti, Mapping), 'argument is not a dictonary-like object'
	mkdirp(dirname(filename))
	# todo: questionable whether the appending of .gz is a good idea; expect `filename` to be the actual filename
	""" http://stackoverflow.com/questions/1447287/format-floats-with-standard-json-module """
	if compressed:
		filename = '%s.gz' % filename
		fh = gzip.open(filename, 'wb+')
		jsn = dumps(dicti, indent = None)
	else:
		fh = open(filename, 'w+')
		jsn = dumps(dicti, indent = 4)
	fh.write(jsn)
	fh.close()


def load_dict(filename, compressed = False, max_age = None):
	"""
	Load a json dictionary file, such as created by :ref: store_dict.

	If compressed is True and the file does not end in .gz, it is automatically appended
	(since store_dict appends it).

	:param filename: (string) path to the file
	:param max_age: (None or number in seconds) if set, file will only be loaded if the file is newer than max_age

	:raises IOError: raied by open() if file can't be opened
	:raises IOAgeError: subclass of IOError, raised if file exists but is older than max_age
	"""
	fail_if_too_old(filename, max_age)
	if compressed:
		if not filename.endswith('.gz'):
			filename = '%s.gz' % filename
		fh = gzip.open(filename, 'rb')
	else:
		fh = open(filename, 'r')
	dicti = loads(fh.read())
	fh.close()
	return dicti






from matplotlib import cm
from matplotlib.pyplot import subplots, show
from numpy import linspace, bincount, hstack, cumsum
from numpy.random import rand, random_integers


def confusion_probs(predictions, labels, ax = None, vmargin = 0.1, equalize = False, cmap = cm.jet):
	"""
	Make a confusion-matrix-like scatter plot
	"""
	N, D = predictions.shape
	bins = bincount(labels)
	assert labels.min() >= 0 and labels.max() < D, 'labels should be between 0 and {0:d}'.format(D - 1)
	assert predictions.min() >= 0 and predictions.max() <= 1, 'predictions should be normalized (to [0, 1])'
	""" Set up graphical stuff. """
	fig = None
	if ax is None:
		fig, ax = subplots(figsize = (8, 8))
		fig.tight_layout()
	if equalize:
		xticks = [x + .5 for x in range(D)]
	else:
		xticks = (float(D) / N) * (cumsum(bins) - bins / 2)
	ax.set_xticks(xticks)
	ax.set_xticklabels(range(D))
	ax.set_xlim([0, D])
	ax.set_yticks([(x + .5) * (1 + vmargin) for x in range(D)])
	ax.set_yticklabels(range(D))
	ax.set_ylim([0, D * (1 + vmargin) - vmargin])
	""" Sort predictions by label. """
	indices = labels.argsort()
	X = predictions[indices, :]
	y = labels[indices]
	""" Add more vertical offset to higher labels. """
	X += linspace(0, D - 1, D) * (1 + vmargin)
	""" Make a new horizontal axis to give all samples equal width. """
	if equalize:
		z = hstack([k + linspace(0, 1, t) for k, t in enumerate(bins)])
	else:
		z = linspace(0, D, N)
	""" Plot the results with class-coding. """
	for q in range(D):
		colors = cmap(y / float(D))
		ax.scatter(z, X[:, q], c = colors, edgecolors = 'none')
	return (fig, ax) if fig else ax


if __name__ == '__main__':
	"""
	Simple demonstration.
	"""
	predictions = rand(10000, 6)
	labels = hstack((random_integers(0, 5, size = (5000,)), random_integers(2, 3, size = (2500,)), [5] * 2500))
	predictions[range(10000), labels] *= 100
	predictions /= predictions.sum(1, keepdims = True)
	confusion_probs(predictions, labels, equalize = True)
	show()






"""
Functions for switching between code styles.
"""

from sys import argv, stderr
from re import sub, search, findall


def docstrings(code, to_double = True):
	"""
	Converts between 3x' and 3x" as long as only one of them occurs in the file.
	"""
	single, double = "'''", '"""'
	if single in code and double in code:
		print('skipping docstrings because both %s and %s appear' % (single, double))
		return code
	if to_double:
		code = code.replace(single, double)
	else:
		code = code.replace(double, single)
	return code


def indentation(code, to_tabs = True):
	"""
	Switch between indentation with tabs or 4 spaces
	"""
	lines = code.splitlines()
	for nr, line in enumerate(lines):
		if to_tabs:
			indent = len(findall(r'^((?:    )*)', line)[0])
			lines[nr] = '\t' * indent + line.lstrip('    ')
		else:
			indent = len(findall(r'^(\t*)', line)[0])
			lines[nr] = '    ' * indent + line.lstrip('\t')
	return '\n'.join(lines)


str = '''
from collections import defaultdict
def group_by(collection, get_property):
	"""
		For a list of items, group them by some property and return tuples of
		that property and all items that have it.

		Similar to :func:`itertools.groupby`, but that requires the items to be ordered by that property;
		more efficient but annoying to use.

		>>> for unique_prop, items in group_by(items, lambda item: item.name).items():
			print '%s: %d' % (unique_prop, len(items))
	"""
	grouped = defaultdict(list)
	for item in collection:
		grouped[get_property(item)].append(item)
	return grouped
"""
	NOTE: if you add something, write a test!
"""'''
print(indentation(str, to_tabs = True))
print(indentation(indentation(str, to_tabs = True), to_tabs = False))
exit()


def first_lines(code, goal_empty_count = 1):
	"""
	Makes sure first lines are empty, after a possible #-prefix.
	"""
	lines = code.splitlines()
	if not lines:
		return code
	""" extract prefix """
	prefix = []
	if lines[0].startswith('#'):
		prefix = [lines.pop(0)]
	""" fix empty lines """
	lines = _prefix_empty_lines(lines, goal_empty_count = goal_empty_count)
	""" construct result """
	return '\n'.join(prefix + lines)


def last_lines(code, goal_empty_count = 3):
	lines = code.splitlines()
	""" fix empty lines """
	lines = _prefix_empty_lines(lines[::-1], goal_empty_count = goal_empty_count + 2)[::-1]
	""" construct result """
	return '\n'.join(lines)


def argument_assignment(code, arg_space = True):
	"""
	Whitespace around = and the like.
	"""
	stderr.write('argument_assignment is not very precise; it will match some constructs such as (a,b)=(1,2) ' + \
		'as well as any strings or comments; it won\'t break code but may create weird formatting\n')
	lines = code.splitlines()
	for nr, line in enumerate(lines):
		copyline = None
		while not copyline == line:
			copyline = line
			if arg_space:
				line = sub(r'\((.*[^ ])=\s*([^ ].*)\)', r'(\1 = \2)', line)
				line = sub(r'\((.*[^ ])\s*=([^ ].*)\)', r'(\1 = \2)', line)
			else:
				line = sub(r'\((.*[^ ])\s+=\s*([^ ].*)\)', r'(\1=\2)', line)
				line = sub(r'\((.*[^ ])\s*=\s+([^ ].*)\)', r'(\1=\2)', line)
		lines[nr] = line
	return '\n'.join(lines)


def imports_together(code):
	"""
	Remove empty lines between imports (like IDE's like to create).
	"""
	lines = code.splitlines()
	import_lines = _find_import_lines(lines)
	if not import_lines:
		return code
	empty_lines = set(range(min(import_lines), max(import_lines) + 1)) - set(import_lines)
	for empty_nr in reversed(sorted(list(empty_lines))):
		lines.pop(empty_nr)
	return '\n'.join(lines)


def _find_import_lines(lines):
	"""
	Internal function that returns the line numbers of the top part import block.

	Any code in the import section will stop the search at that point.
	"""
	in_imports, import_lines = False, []
	for nr, line in enumerate(lines):
		if search(r'import\s+[\w\._]+', line) or search(r'from\s+[\w\._]+\s+import\s+[\w\._]+', line):
			in_imports = True
			import_lines.append(nr)
		elif not line.strip():
			continue
		else:
			if in_imports:
				break
	return import_lines


def _prefix_empty_lines(lines, goal_empty_count):
	"""
	Internal function to prepend empty lines (no prefix considerations).
	"""
	""" find current empty line count """
	current_empty_count = 0
	for line in lines:
		if line.strip():
			break
		current_empty_count += 1
	""" insert newlines if too few """
	lines = [''] * (goal_empty_count - current_empty_count) + lines
	""" remove newlines if too many """
	if current_empty_count > goal_empty_count:
		lines = lines[current_empty_count - goal_empty_count:]
	""" done! """
	return lines


def _print_code(code):
	print('+' * 20)
	for nr, line in enumerate(code.splitlines()):
		print('%3d\t%s' % (nr + 1, line.replace('\t', '~   ')))
	print('+' * 20)


def change_settings(code, to_tabs = None, to_double = None, start_empty = None, end_empty = None, arg_space = None):
	"""
	Apply various settings to a code string.
	"""
	if to_tabs is not None:
		code = indentation(code, to_tabs = to_tabs)
	if to_double is not None:
		code = docstrings(code, to_double = to_double)
	if start_empty is not None:
		code = first_lines(code, goal_empty_count = start_empty)
	if end_empty is not None:
		code = last_lines(code, goal_empty_count = end_empty)
	if arg_space is not None:
		code = argument_assignment(code, arg_space = arg_space)
	code = imports_together(code)
	return code


def change_file(filepath, change = False, to_tabs = None, to_double = None, start_empty = None, end_empty = None,
		arg_space = None):
	"""
	Apply various settings to a file if change = True (otherwise prints only).
	"""
	with open(filepath, 'r') as fh:
		code = fh.read()
		_print_code(code)
	code = change_settings(code, to_tabs = to_tabs, to_double = to_double, start_empty = start_empty,
		end_empty = end_empty, arg_space = arg_space)
	if change:
		with open(filepath, 'w') as fh:
			fh.write(code)
	else:
		_print_code(code)


def pep_files(filepaths, change = False):
	"""
	Convert somewhat to pep-compliant format for all the files in list filepaths.
	"""
	for filepath in filepaths:
		change_file(filepath, change = change, to_double = False, arg_space = False)


def mark_style_files(filepaths, change = False):
	"""
	Convert somewhat to pep-compliant format for all the files in list filepaths.
	"""
	for filepath in filepaths:
		change_file(filepath, change = change, to_double = True, arg_space = True, start_empty = 1, end_empty = 3)


if __name__ == '__main__':
	if len(argv) < 3:
		print('first argument should be the style; "pep" or "mark"')
		print('provide at least one file path of a python file (no directories)')
	filepaths = argv[2:]
	if argv[1] == 'pep':
		pep_files(filepaths)
	elif argv[1] == 'mark':
		mark_style_files(filepaths)
	else:
		print('first argument should be the style; "pep" or "mark" (got %s)' % argv[1])





from collections import defaultdict


def group_by(collection, get_property):
	"""
	For a list of items, group them by some property and return tuples of
	that property and all items that have it.

	Similar to :func:`itertools.groupby`, but that requires the items to be ordered by that property; \
	more efficient but annoying to use.

	>>> for unique_prop, items in group_by(items, lambda item: item.name).items():
		print '%s: %d' % (unique_prop, len(items))
	"""
	grouped = defaultdict(list)
	for item in collection:
		grouped[get_property(item)].append(item)
	return grouped


def without(iterable, remove_indices):
	"""
	Returns an iterable for a collection or iterable, which returns all items except the specified indices.
	"""
	if not hasattr(remove_indices, '__iter__'):
		remove_indices = {remove_indices}
	else:
		remove_indices = set(remove_indices)
	for k, item in enumerate(iterable):
		if k in remove_indices:
			continue
		yield item


def make_unique_strs(labels):
	"""
	Add numerical postfix to every string in the list to make all of them unique.
	"""
	unique_labels = []
	used_labels = set()
	counts = defaultdict(int)
	for label in labels:
		counts[label] += 1
	for label, cnt in counts.items():
		if cnt > 1:
			used_labels.add(label)
	for baselabel in labels:
		label, k = baselabel, 1
		while label in used_labels:
			label = '{0:}-{1:d}'.format(baselabel, k)
			k += 1
		used_labels.add(label)
		unique_labels.append(label)
	return unique_labels

